#!/bin/sh
TOPDIR=../../../../
BUILD=""
PLATFORM="s3c64xx"
KERNELDIR="${TOPDIR}/linux-kernel"
KERNELCONF="seoul_flasher_defconfig"
UBOOTDIR="${TOPDIR}/u-boot"



die() {
  echo "$0: ERROR: $1 $2!"
  exit 1
}

usage() {
  cat << __EOT__
Usage: $0 <seoul|seoul_mmc|seoul2_mmc> [clean | check | build | rebuild]
* clean - delete all build products
__EOT__
  exit 1
}

clean() {
  make PLATFORM=${PLATFORM} clean || die "Could not clean"
  cd ${KERNELDIR}
  ./buildkernel.sh clean config=${KERNELCONF} || die "Could not build flasher kernel"
  cd -  
  cd ${UBOOTDIR}
  ./build.sh clean || die "Could not build flasher u-boot"
  cd -
  rm -fr ${PLATFORM}-bin
  rm -fr ${PLATFORM}-disk
  rm -fr files
}

build() {
  mkdir -p ${PLATFORM}-bin
  make PLATFORM=${PLATFORM} || die "Could not build initramfs contents"
  cd ${KERNELDIR}
  ./buildkernel.sh ${BUILD} config=${KERNELCONF} || die "Could not build flasher kernel"
  cd -
  cp ${KERNELDIR}/arch/arm/boot/zImage ${PLATFORM}-bin/zImage.bin
  cp ${KERNELDIR}/arch/arm/boot/uImage ${PLATFORM}-bin/uImage
  cd ${UBOOTDIR}
  ./build.sh ${BUILD} ${UBOOTCONF} || die "Could not build flasher u-boot"
  cd -
  cp ${UBOOTDIR}/u-boot ${PLATFORM}-bin/u-boot.elf
  cp ${UBOOTDIR}/u-boot-movi.bin ${PLATFORM}-bin/u-boot-movi.bin
  cp ${UBOOTDIR}/bootstrap.bin ${PLATFORM}-bin/bootstrap.bin
  echo "Flasher Initramfs built successfully"
  ls -al ${PLATFORM}-bin

  mkdir -p files
  cp ${PLATFORM}-bin/uImage files/
  
  mkdir -p ${PLATFORM}-disk
  cd ${PLATFORM}-disk
  PATH=/sbin:$PATH ../../../../../Tools/Linux/bin/ttimage -c ../parttbl_flash.xml -o disk_image.img  -l 0xdf,0xc7e00000  -j 0xdf,0xc7e00000 -v 0xdf,1000283653 -s 7340032
  cd -

  cp ${PLATFORM}-bin/u-boot-movi.bin ${PLATFORM}-disk/
}

#######################################
# Main program starts here

while test -n "$1"; do
  case "$1" in
  seoul)
    PLATFORM="s3c64xx"
    KERNELCONF="seoul_flasher_defconfig"
    KERNELDIR="${TOPDIR}/linux-kernel"
    UBOOTCONF="seoul_flasher"
    UBOOTDIR="${TOPDIR}/u-boot"
    ;;
  seoul_mmc)
    PLATFORM="s3c64xx"
    KERNELCONF="seoul_flasher_defconfig"
    KERNELDIR="${TOPDIR}/linux-kernel"
    UBOOTCONF="seoul_mmc0"
    UBOOTDIR="${TOPDIR}/u-boot"
    ;;
  seoul2_mmc)
    PLATFORM="s3c64xx"
    KERNELCONF="seoul2_flasher_defconfig"
    KERNELDIR="${TOPDIR}/linux-kernel"
    UBOOTCONF="seoul2_flasher"
    UBOOTDIR="${TOPDIR}/u-boot"
    ;;
  build)
    opt_build="y"
    ;;
  rebuild)
    opt_rebuild="y"
    ;;  
  check)
    opt_check="y"
    ;;
  clean)
    opt_clean="y"
    ;;
  help)
    usage
    ;;
  *)
    echo "$0: Unknown argument $1,"
    usage
    ;;
  esac
  shift
done

if test "${opt_clean}" = "y"; then
  clean
  exit 1
fi

if test "${opt_check}" = "y"; then
  check
  exit 1
fi

if test "${opt_rebuild}" = "y" -o "${opt_build}" = "y"; then
  if test "${opt_rebuild}" = "y"; then
    BUILD="rebuild"
  fi
  build
  exit 1
fi


