/*
 * splash
 */
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include <linux/fb.h>

#include "splash.h"

int open_fb(struct fb_data *fb_d, char *fb_dev_name)
{

    /* open the frame buffer */
    fb_d->fb_handle = open(fb_dev_name, O_RDWR);

    if (fb_d->fb_handle == -1) {
	perror("open framebuffer");
	return -1;
    }

    /* read  the fixed and variable framebuffer info */
    if (ioctl
	(fb_d->fb_handle, FBIOGET_FSCREENINFO,
	 &fb_d->fixed_screen_info) < 0) {
	perror("get screen info");
	close(fb_d->fb_handle);
	return -2;
    }

    if (ioctl
	(fb_d->fb_handle, FBIOGET_VSCREENINFO,
	 &fb_d->variable_screen_info) < 0) {
	perror("get screen info");
	close(fb_d->fb_handle);
	return -3;
    }
    fb_d->fb_buffer =
	mmap(NULL, fb_d->fixed_screen_info.smem_len,
	     PROT_READ | PROT_WRITE, MAP_FILE | MAP_SHARED,
	     fb_d->fb_handle, 0);

    if (fb_d->fb_buffer == MAP_FAILED) {
	close(fb_d->fb_handle);
	return -4;
    }
    fb_d->curent_page = 0;
    return 0;			/* success */
}

void close_fb(struct fb_data *fb_d)
{
    munmap(fb_d->fb_buffer, fb_d->fixed_screen_info.smem_len);
    close(fb_d->fb_handle);
}

void put_pixel(struct fb_data *fb_d, int x, int y, unsigned char red,
	       unsigned char green, unsigned char blue)
{
    int offset =
	(fb_d->variable_screen_info.yoffset *
	 fb_d->fixed_screen_info.line_length) +
	y * fb_d->fixed_screen_info.line_length +
	(x * fb_d->variable_screen_info.bits_per_pixel >> 3);


    if (fb_d->variable_screen_info.bits_per_pixel == 16) {
	*(volatile unsigned short *) (fb_d->fb_buffer + offset) =
	    ((red >> 3) << fb_d->variable_screen_info.red.offset) |
	    ((green >> 2) << fb_d->variable_screen_info.green.offset) |
	    ((blue >> 3) << fb_d->variable_screen_info.blue.offset);
    }
}

void draw_rle_file(struct fb_data *fb_d, char *name, int align, int upsidedown)
{
    struct image_header header;
    unsigned char length;
    unsigned short color;
    unsigned int counter;
    unsigned int l;
    unsigned int base_x, base_y;

    unsigned char red;
    unsigned char green;
    unsigned char blue;

    FILE *fp = fopen(name, "rb");
    if (!fp) {
	printf("Failed to open RLE file\n");
	return;
    }

    fread(&header, sizeof(struct image_header), 1, fp);

    if (fb_d->variable_screen_info.xres < header.width) {
	printf("Warning, croping image width to %d pixels\n", fb_d->variable_screen_info.xres);
	header.width = fb_d->variable_screen_info.xres;
     }

     if (fb_d->variable_screen_info.yres < header.height) {
	printf("Warning, croping image height to %d pixels\n", fb_d->variable_screen_info.yres);
	header.height = fb_d->variable_screen_info.yres;
    }

    printf("Image is w,h,bpp=%i ,%i ,%i\n", header.width, header.height,
	   header.bpp);
    if (align == IMAGE_ALIGN_CENTER) {
	base_x = (fb_d->variable_screen_info.xres - header.width) / 2;
	base_y = (fb_d->variable_screen_info.yres - header.height) / 2;
    } else {
	base_x = 0;
	base_y = 0;
    }
    counter = 0;
    while (counter < header.width * header.height) {
	fread(&length, sizeof(unsigned char), 1, fp);;
	fread(&color, sizeof(unsigned short int), 1, fp);
	red = ((color >> 11) & 0x1f) << 3;
	green = ((color >> 5) & 0x1f) << 3;
	blue = (color & 0x1f) << 3;

	if (upsidedown)
	{
		for (l = 0; l < length; l++) {
			put_pixel(fb_d, header.width - (base_x + (counter % header.width)),
				header.height - (base_y + (counter / header.width)), red, green, blue);
			counter++;
		}
	}
	else
	{
		for (l = 0; l < length; l++) {
			put_pixel(fb_d, base_x + (counter % header.width),
				base_y + (counter / header.width), red, green, blue);
			counter++;
		}
	}
    }
    fclose(fp);
}

void flip(struct fb_data *fb_d)
{
    if (fb_d->perform_flip) {
	fb_d->curent_page = (fb_d->curent_page) ? 0 : 1;
	fb_d->variable_screen_info.yoffset =
	    fb_d->curent_page * fb_d->variable_screen_info.yres;
	ioctl(fb_d->fb_handle, FBIOPAN_DISPLAY,
	      &fb_d->variable_screen_info);
    }
}

